package App::Ack::Filter;

use strict;
use warnings;

use App::Ack ();
use App::Ack::Filter::Inverse ();

my %filter_types;


sub create_filter {
    my ( undef, $type, @args ) = @_;

    if ( my $package = $filter_types{$type} ) {
        return $package->new(@args);
    }
    my $allowed_types = join( ', ', sort keys %filter_types );
    App::Ack::die( "Unknown filter type '$type'.  Type must be one of: $allowed_types." );
}


sub register_filter {
    my ( undef, $type, $package ) = @_;

    $filter_types{$type} = $package;

    return;
}


sub invert {
    my ( $self ) = @_;

    return App::Ack::Filter::Inverse->new( $self );
}


sub is_inverted {
    return 0;
}


sub to_string {
    return '(unimplemented to_string)';
}


sub inspect {
    my ( $self ) = @_;

    return ref($self);
}

1;
